import copy

from digExtractor.extractor import Extractor
import social_extractor

class SocialMediaIdExtractor(Extractor):

    def __init__(self):
        self.renamed_input_fields = 'tokens'
        self.metadata = {"extractor": "SocialMediaIdExtractor"}
        self.extractor = social_extractor.SocialExtractor(high_recall=True)

    def extract(self, doc):
        try:
            handles = None
            if 'tokens' in doc:
                tokens = doc["tokens"]
                handles = self.extractor.extract(tokens)
            return handles
        except:
            return None

    def get_metadata(self):
        """Returns a copy of the metadata that characterizes this extractor"""
        return copy.copy(self.metadata)

    def set_metadata(self, metadata):
        """Overwrite the metadata that characterizes this extractor"""
        self.metadata = metadata
        return self

    def get_renamed_input_fields(self):
        """Return a scalar or ordered list of fields to rename to"""
        return self.renamed_input_fields



import enchant

TWITTER = 'twitter'
INSTAGRAM = 'instagram'
FACEBOOK = 'facebook'
WECHAT = 'wechat'
WHATSAPP = 'whatsapp'
GOOGLE = 'google'
LINKEDIN = 'linkedin'
LINKDIN = 'linkdin'
SOCIAL_MEDIAS = [TWITTER, INSTAGRAM, FACEBOOK, WECHAT, WHATSAPP, GOOGLE, LINKEDIN, LINKDIN]
class SocialExtractor:
    def __init__(self, high_recall = True, forward_limit=11):
        self.is_high_recall = high_recall
        self.dictionary_en = enchant.Dict("en_US")
        self.dictionary_fr = enchant.Dict("fr_FR")
        self.forward_limit = forward_limit

    def extract(self, tokens):
        tokens = map(lambda x:x.lower(),tokens)
        #print tokens
        handles = dict()

        handle = self.get_handle_after_social_media(tokens, TWITTER)
        handles['twitter'] = handle
        handle = self.get_handle_after_social_media(tokens, INSTAGRAM)
        handles['instagram'] = handle
        return handles

    def get_handle_after_social_media(self, tokens, social_media=TWITTER):
        handle = None
        #print tokens
        if(social_media in tokens):
            social_media_index = tokens.index(social_media)
            handle_index = social_media_index + 1
            limit = min(social_media_index + self.forward_limit, len(tokens))
            check_dictionary = True
            while(handle_index < limit):
                word_at_index = tokens[handle_index]
                #print "Current Word:"+word_at_index
                if(word_at_index == u"@"):
                    if(handle_index + 1 < limit):
                        return tokens[handle_index+1]
                elif(check_dictionary and is_valid_handle(word_at_index, social_media)):
                    #Word is a potential handle
                    if(word_at_index.isalpha()):
                        check_dictionary = False
                        if(not word_at_index in SOCIAL_MEDIAS and not self.dictionary_en.check(word_at_index) 
                            and not self.dictionary_fr.check(word_at_index)):
                            return word_at_index
                handle_index += 1
        return handle

def is_valid_handle(handle, social_media):
    if(social_media == INSTAGRAM):
        #Max 30 chars which are letters, numbers, underscores and periods
        if(len(handle) <= 30):
            handle = handle.replace("_","")
            handle = handle.replace(".","")
            if(handle.isalnum()):
                return True
    if(social_media == TWITTER):
        #Max 15 chars which are letter, numbers, underscores
        if(len(handle) <= 15):
            handle = handle.replace("_","")
            if(handle.isalnum()):
                return True
    return False



import os
import sys
import codecs

import unittest

import json
from digExtractor.extractor_processor import ExtractorProcessor
from digSocialMediaIdExtractor import socialmedia_id_extractor

class TestSocialMediaExtractor(unittest.TestCase):

    def test_socialmedia_id_extractor(self):
        doc = {"tokens":["adair","location",":","escorts","missouri","escorts","kansas",
        "city","escorts","adair","my","information","follow","me","on","twitter","@","DiamondSquirt","location",":","kansas","city","escorts","type",":"]}

        #print doc["tokens"][0]["result"][0]["value"]
        extractor = socialmedia_id_extractor.SocialMediaIdExtractor()
        extractor_processor = ExtractorProcessor().set_input_fields('tokens').set_output_field('social_media_ids').set_extractor(extractor)

        updated_doc = extractor_processor.extract(doc)

        #print updated_doc

        self.assertEquals(updated_doc['social_media_ids'][0]['result']['value'], {'twitter': 'diamondsquirt', 'instagram': None})

    def test_missing_tokens(self):
        doc = {"tokens":[]}
        extractor = socialmedia_id_extractor.SocialMediaIdExtractor()
        extractor_processor = ExtractorProcessor().set_input_fields('tokens').set_output_field('social_media_ids').set_extractor(extractor)

        updated_doc = extractor_processor.extract(doc)

        #print updated_doc

        self.assertEquals(updated_doc, doc) 

    def test_empty_tokens(self):
        doc = {}
        extractor = socialmedia_id_extractor.SocialMediaIdExtractor()
        extractor_processor = ExtractorProcessor().set_input_fields('tokens').set_output_field('social_media_ids').set_extractor(extractor)

        updated_doc = extractor_processor.extract(doc)

        #print updated_doc

        self.assertEquals(updated_doc, {})       

if __name__ == '__main__':
    unittest.main()
